import 'package:args/args.dart';
import 'package:mocktail/mocktail.dart';
import 'package:test/test.dart';
import 'package:widgetbook_git/widgetbook_git.dart';

import '../../bin/ci_parser/ci_parser.dart';
import '../../bin/models/models.dart';
import '../mocks/command_mocks.dart';

const repositoryName = 'widgetbook';
const actorName = 'John Doe';

void main() {
  late CiParserRunner sut;
  late ArgResults argResults;
  late GitDir gitDir;

  setUp(() async {
    argResults = MockArgResults();
    gitDir = MockGitDir();

    sut = CiParserRunner(
      argResults: argResults,
      gitDir: gitDir,
    );
  });

  group('$CiParserRunner', () {
    test(
      'can accept $ArgResults',
      () async {
        expect(sut.argResults, isA<MockArgResults>());
      },
    );

    test(
      'can accept $MockGitDir',
      () async {
        expect(sut.gitDir, isA<MockGitDir>());
      },
    );

    test(
      'returns $CiArgs from $LocalParser',
      () async {
        when(() => gitDir.getActorName())
            .thenAnswer((_) => Future.value(actorName));

        when(() => gitDir.getRepositoryName())
            .thenAnswer((_) => Future.value(repositoryName));

        final sut = CiParserRunner(
          argResults: argResults,
          gitDir: gitDir,
        );

        final ciArgs = await sut.getParser()?.getCiArgs();

        expect(ciArgs?.actor, actorName);
        expect(ciArgs?.repository, repositoryName);
        expect(ciArgs?.vendor, 'Local');
      },
    );

    test(
      'returns $CiArgs from $GitLabParser',
      () async {
        final ciArgs = await sut.getParser()?.getCiArgs();

        expect(ciArgs?.actor, actorName);
        expect(ciArgs?.repository, repositoryName);
        expect(ciArgs?.vendor, 'GitLab');
      },
      skip: 'looking for a better way to set platform environment variables',
    );
    test(
      'returns $CiArgs from $GitHubParser',
      () async {
        final ciArgs = await sut.getParser()?.getCiArgs();

        expect(ciArgs?.actor, actorName);
        expect(ciArgs?.repository, repositoryName);
        expect(ciArgs?.vendor, 'GitHub');
      },
      skip: 'looking for a better way to set platform environment variables',
    );

    test(
      'returns $CiArgs from $AzureParser',
      () async {
        final ciArgs = await sut.getParser()?.getCiArgs();

        expect(ciArgs?.actor, actorName);
        expect(ciArgs?.repository, repositoryName);
        expect(ciArgs?.vendor, 'Azure');
      },
      skip: 'looking for a better way to set platform environment variables',
    );

    test(
      'returns $CiArgs from $BitbucketParser',
      () async {
        final ciArgs = await sut.getParser()?.getCiArgs();

        expect(ciArgs?.actor, actorName);
        expect(ciArgs?.repository, repositoryName);
        expect(ciArgs?.vendor, 'Bitbucket');
      },
      skip: 'looking for a better way to set platform environment variables',
    );
  });
}
